# -*- coding: utf-8 -*-
#
# New BSD license
#
# Copyright (c) DR0ID
# This file 'compatibility.py' is part of HG_pyknic
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL DR0ID BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
This module ensures that framework functions and classes that have changed between 2.7 and 3.5+ can be used in
the code in the same way.
"""
from __future__ import print_function, division

import logging
import sys

logger = logging.getLogger(__name__)
logger.debug("importing...")

__version__ = '1.0.0.0'

# for easy comparison as in sys.version_info but digits only
__version_info__ = tuple([int(_d) for _d in __version__.split('.')])

__author__ = "DR0ID"
__email__ = "dr0iddr0id {at} gmail [dot] com"
__copyright__ = "DR0ID @ 2016"
__credits__ = ["DR0ID"]  # list of contributors
__maintainer__ = "DR0ID"
__license__ = "New BSD license"

__all__ = [
    "check_for_supported_python_version",
    "is_py2",
    "is_py3",
    "StringIO",
]  # list of public visible parts of this module

_version_info = sys.version_info
_version_str = ".".join([str(_v) for _v in _version_info])
_missing_or_unsupported_error = "Missing import or unsupported python version '{0}' for {1}".format(
    _version_str, "'{0}'")


def check_for_supported_python_version():
    if _version_info[0] == 2:
        if _version_info[1] < 7:
            raise SystemError("Unsupported python v{0}! Use python v2.7 or later.".format(_version_str))


check_for_supported_python_version()

is_py2 = _version_info[0] == 2
is_py3 = _version_info[0] == 3

if is_py2:
    # noinspection PyCompatibility
    from StringIO import StringIO
elif is_py3:
    from io import StringIO
else:
    raise SystemError(_missing_or_unsupported_error.format("StringIO"))

if is_py2:
    # noinspection PyShadowingBuiltins
    input = raw_input
elif is_py3:
    # noinspection PyShadowingBuiltins
    input = input
else:
    raise SystemError(_missing_or_unsupported_error.format("input"))

logger.debug("imported")
